/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#ifndef _dview_h_
#define _dview_h_

#include <QUrl>
#include <QtDebug>
#include <QWebView>
#include <QWebFrame>
#include <QMouseEvent>
#include <QWebElement>

#include "dwebpage.h"
#include "dftpbrowser.h"
#include "dwebhistory.h"
#include "dfilemanager.h"
#include "dwebhistoryitem.h"

class dwebview: public QWebView
{
  Q_OBJECT

 public:
  dwebview(QWidget *parent):QWebView(parent)
  {
    m_allowPopup = false;
  }

  bool allowPopup(void) const
  {
    return m_allowPopup;
  }

  void resetAllowPopup(void)
  {
    m_allowPopup = false;
  }

 private:
  bool m_allowPopup;
  void mousePressEvent(QMouseEvent *event);
};

class dview: public QStackedWidget
{
  Q_OBJECT

 public:
  dview(QWidget *parent);
  ~dview();
  int progress(void) const;
  bool isDir(void) const;
  bool isFtp(void) const;
  bool isLoaded(void) const;
  bool canGoBack(void) const;
  bool canGoForward(void) const;
  QUrl url(void) const;
  void back(void);
  void load(const QUrl &url);
  void stop(void);
  void print(QPrinter *printer);
  void reload(void);
  void forward(void);
  void setFocus(void);
  void setTabAction(QAction *action);
  QIcon icon(void) const;
  QAction *tabAction(void) const;
  QString html(void) const;
  QString title(void) const;
  dwebpage *page(void) const;
  QWebFrame *currentFrame(void) const;
  dwebhistoryitem currentItem(void) const;
  QList<dwebhistoryitem> backItems(const int n) const;
  QList<dwebhistoryitem> forwardItems(const int n) const;

 private:
  int m_percentLoaded;
  bool m_pageLoaded;
  QUrl selectedUrl;
  QUrl selectedImageUrl;
  QIcon m_webViewIcon;
  QAction *m_action;
  dwebview *webView;
  QWebView *directoryView;
  dftpbrowser *ftpBrowser;
  dwebhistory *m_history;
  dfilemanager *fileManager;
  QNetworkAccessManager *m_faviconManager;
  int indexForUrl(const QUrl &url) const;
  void enterEvent(QEvent *event);
  void downloadFavicon(const QUrl &url);

 private slots:
  void slotPaste(void);
  void slotReload(void);
  void slotAdBlock(void);
  void slotSaveLink(void);
  void slotSaveLink(const QUrl &url);
  void slotSaveImage(void);
  void slotSslErrors(QNetworkReply *reply, const QList<QSslError> &errors);
  void slotViewImage(void);
  void slotUrlChanged(const QUrl &url);
  void slotIconChanged(void);
  void slotLinkClicked(const QUrl &url);
  void slotLoadStarted(void);
  void slotClearHistory(void);
  void slotLoadFinished(bool ok);
  void slotLoadProgress(int progress);
  void slotTitleChanged(const QString &title);
  void slotCopyLinkLocation(void);
  void slotCopyLinkLocation(const QUrl &url);
  void slotCopySelectedText(void);
  void slotOpenLinkInNewTab(void);
  void slotOpenLinkInNewTab(const QUrl &url);
  void slotSelectionChanged(void);
  void slotCopyImageLocation(void);
  void slotOpenImageInNewTab(void);
  void slotOpenLinkInNewWindow(void);
  void slotOpenLinkInNewWindow(const QUrl &url);
  void slotIconDownloadFinished(void);
  void slotOpenImageInNewWindow(void);
  void slotHandleUnsupportedContent(QNetworkReply *reply);
  void slotCustomContextMenuRequested(const QPoint &pos);

 signals:
  void saveUrl(const QUrl &url);
  void copyLink(const QUrl &url);
  void saveFile(const QString &fileName, QNetworkReply *reply);
  void viewImage(const QUrl &url);
  void urlChanged(const QUrl &url);
  void iconChanged(void);
  void loadStarted(void);
  void viewEntered(void);
  void loadFinished(bool);
  void loadProgress(const int);
  void titleChanged(const QString &title);
  void openLinkInNewTab(const QUrl &url);
  void selectionChanged(const QString &text);
  void openLinkInNewWindow(const QUrl &url);
};

#endif
